from typing import List

from RsInstrument import *
from GFM314_Rx_tests.General.GFM314_utilities import *


class RxTestsBase:
    def __init__(self, resource_name: str, resource_name_additional_cw_generator=None):
        try:
            self.io = RsInstrument(resource_name, True, True)
            if resource_name_additional_cw_generator is not None:
                self.additional_cw_generator = RsInstrument(resource_name_additional_cw_generator, True, True)
        except ResourceError as e:
            raise RsException(e.args[0])
        self.io.visa_timeout = 5000

    def instrument_settings(self, trigger_config, marker_config):
        """
        This function sets instrument related settings.
        :param trigger_config: Trigger configuration:  Armed Auto ('AAUT') or unchanged ('UNCH')
        :param marker_config: Marker configuration: Radio frame start ('FRAM') or unchanged ('UNCH')
        :return: Instrument status
        """
        if trigger_config is not None:
            self.io.write_str(f':BB:NR5G:TCW:TRIG {trigger_config}')
        if marker_config is not None:
            self.io.write_str(f':BB:NR5G:TCW:MARK {marker_config}')
        return self.io.query_str(':syst:err?')

    def reset(self):
        """
        This function resets the connected instrument
        :return: Instrument status
        """
        self.io.write_str('*RST')
        return self.io.query_str(':syst:err?')

    def tcw_apply(self):
        """
        This function applies the signal configuration (TCW)
        :return: Instrument status
        """
        self.io.write_str(':BB:NR5G:TCW:APPL')
        return self.io.query_str(':syst:err?')

    def output_on(self, output: int):
        """
        This function switches the output on
        :param output: Output number
        :return: Instrument status
        """
        self.io.write_str(f':OUTPut{output}:STAT 1')
        return self.io.query_str(':syst:err?')

    def output_off(self, output: int):
        """
        This function switches the output off
        :param output: Output number
        :return: Instrument status
        """
        self.io.write_str(f':OUTPut{output}:STAT 0')
        return self.io.query_str(':syst:err?')

    def close(self):
        """
        This function closes the instrument connection
        """
        self.io.close()

    def query_status(self):
        """
        This function queries the status of the connected instrument
        :return: Instrument status
        """
        return self.io.query_str(':syst:err?')

    def query_errors(self):
        """
        This function queries all errors of the connected instrument
        :return: List of errors
        """
        return self.io.query_all_errors()

    def query_awgn_power(self):
        """
        This function returns the AWGN power level in dBm
        :return: AWGN power level [dBm]
        """
        awgn_query = self.io.query_str(':BB:NR5G:TCW:AWGN:PLEV?').split(' ')
        awgn_power = float(awgn_query.pop(0)[1:])
        return awgn_power

    def query_awgn_bandwidth(self):
        """
        This function returns the AWGN bandwdith in MHz
        :return: AWGN bandwidht [MHz]
        """
        awgn_query = self.io.query_str(':BB:NR5G:TCW:AWGN:PLEV?').split(' ')
        awgn_bandwdith = float(awgn_query.pop(3))
        return awgn_bandwdith

    def query_frc(self):
        """
        This function returns the used fixed reference channel
        :return: Fixed reference channel
        """
        return self.io.query_str(':BB:NR5G:TCW:WS:FRC:TYPE?')

    def query_power(self):
        """
        This function returns the power level in dBm
        :return: Power level [dBm]
        """
        return self.io.query_float(':BB:NR5G:TCW:WS:PLEV?')

    def _check_device_suitability(self, needed: List[str]):
        """This function returns missing, required and already existing devices options."""
        missing = []
        for option in needed:
            if option not in self.io.instrument_options:
                missing.append(option)
        if len(missing) > 0:
            raise RsException(
                f'Some of the required options are missing.\nRequired: {", ".join(needed)}\n'
                f'Missing: {", ".join(missing)}\nAll options: {", ".join(self.io.instrument_options)}')

    def hardcopy(self):
        """
        Save hardcopy of connected device to project directory
        """
        self.io.write_str('HCOP:DEV:LANG1 PNG')
        path = r'c:\R_S\Instr\user\hardcopy.png'
        self.io.write_str("MMEM:NAME '" + path + "'")
        self.io.write_str_with_opc('HCOP:IMM1')
        self.io.read_file_from_instrument_to_pc(path, r'hardcopy.png')
